from enum import Enum


class TerminalColors(Enum):
    Red = "\033[91m"
    Green = "\033[92m"
    Blue = "\033[94m"
    Cyan = "\033[96m"
    White = "\033[97m"
    Yellow = "\033[93m"
    Magenta = "\033[95m"
    Grey = "\033[90m"
    Black = "\033[90m"
    Default = "\033[99m"

    # special colors
    Warning = "\033[93m"
    Error = "\033[91m"
    Success = "\033[92m"


class TerminalStyling(Enum):
    Header = "\033[95m"
    EndCharacter = "\033[0m"
    Bold = "\033[1m"
    Underline = "\033[4m"


__all__ = ["TerminalColors", "TerminalStyling"]



import os
import posixpath
from urllib.parse import unquote, urlsplit


def filename_from_url(url):
    """Return basename corresponding to url.
    >>> print(url2filename('http://example.com/path/to/file%C3%80?opt=1'))
    fileÀ
    >>> print(url2filename('http://example.com/slash%2fname')) # '/' in name
    Traceback (most recent call last):
    ...
    ValueError
    """

    urlpath = urlsplit(url).path
    basename = posixpath.basename(unquote(urlpath))
    if (
        os.path.basename(basename) != basename
        or unquote(posixpath.basename(urlpath)) != basename
    ):
        raise ValueError  # reject '%2f' or 'dir%5Cbasename.ext' on Windows

    return basename


__all__ = ["filename_from_url"]



from os import path

from nwon_baseline.file_helper.file import file_extension_from_path


class WrongFilePath(Exception):
    pass


def path_to_write_pydantic_model(file_path: str) -> str:
    suffix = file_extension_from_path(file_path)

    if suffix != ".json":
        splitted_file_name = path.basename(file_path).split(".")
        directory = path.dirname(file_path)

        if len(splitted_file_name) > 2:
            raise WrongFilePath(
                f"The file type of {path.basename(file_path)} "
                + "needs to be json or it can't contain a dot."
            )

        if len(splitted_file_name) == 2:
            file_path = path.join(directory, f"{splitted_file_name[0]}.json")

    return file_path



import subprocess
import sys
from typing import List


def execute_command_and_print(command: List[str]) -> None:
    print(execute_command(command))


def execute_command(command: List[str]) -> str:
    try:
        result = subprocess.run(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            check=True,
        )
        output = result.stdout
        return output
    except subprocess.CalledProcessError as error:
        output = error.stdout
        error_output = error.stderr

        print(f"Command '{command}' failed with exit code {error.returncode}.")
        print("Output:\n", output)
        print("Error output:\n", error_output)
        sys.exit(1)


__all__ = [
    "execute_command",
    "execute_command_and_print",
]



from typing import Iterable, Union


def unique_list(
    list_to_check: Union[
        Iterable[Union[str, int, float, bool, bytes]],
        Iterable[str],
        Iterable[int],
        Iterable[float],
        Iterable[bool],
        Iterable[bytes],
    ]
):
    """
    Unifies items in a list.
    """

    seen = set()
    seen_add = seen.add
    return [x for x in list_to_check if not (x in seen or seen_add(x))]



from nwon_baseline.typings import TerminalColors, TerminalStyling


def print_red(text: str):
    print_color(text, TerminalColors.Red)


def print_green(text: str):
    print_color(text, TerminalColors.Green)


def print_blue(text: str):
    print_color(text, TerminalColors.Blue)


def print_cyan(text: str):
    print_color(text, TerminalColors.Cyan)


def print_white(text: str):
    print_color(text, TerminalColors.White)


def print_yellow(text: str):
    print_color(text, TerminalColors.Yellow)


def print_magenta(text: str):
    print_color(text, TerminalColors.Magenta)


def print_grey(text: str):
    print_color(text, TerminalColors.Grey)


def print_black(text: str):
    print_color(text, TerminalColors.Black)


def print_default(text: str):
    print_color(text, TerminalColors.Default)


def print_warning(text: str):
    print_color(text, TerminalColors.Warning)


def print_error(text: str):
    print_color(text, TerminalColors.Error)


def print_success(text: str):
    print_color(text, TerminalColors.Success)


def print_color(text: str, color: "TerminalColors") -> None:
    print(f"{color.value}{text}{TerminalStyling.EndCharacter.value}")



from typing import TYPE_CHECKING

from nwon_baseline.typings import TerminalStyling

if TYPE_CHECKING:
    from nwon_baseline.typings import TerminalColors


def print_with_color_and_style(
    text: str, color: "TerminalColors", style: "TerminalStyling"
) -> None:
    print(f"{color.value}{style.value}{text}{TerminalStyling.EndCharacter.value}")


def print_with_style(text: str, style: "TerminalStyling") -> None:
    print(f"{style.value}{text}{TerminalStyling.EndCharacter.value}")



from nwon_baseline.pydantic.pydantic_model_to_dict import pydantic_model_to_dict
from nwon_baseline.pydantic.save_pydantic_model_instance import (
    save_pydantic_model_instance_as_json,
)
from nwon_baseline.pydantic.save_pydantic_model_schema import save_pydantic_model_schema

__all__ = [
    "save_pydantic_model_instance_as_json",
    "save_pydantic_model_schema",
    "pydantic_model_to_dict",
]



from pydantic import BaseModel

from nwon_baseline.pydantic.path_to_write_pydantic_model import (
    path_to_write_pydantic_model,
)


def save_pydantic_model_instance_as_json(
    pydantic_model: BaseModel, file_path: str
) -> str:
    """
    Saves the Pydantic Model instance as a json file.

    Expects the file_path to end with '.json'.
    If the suffix is not json we take the basename of the file_path and attach .json
    """

    file_path = path_to_write_pydantic_model(file_path)

    with open(file_path, "w+", encoding="utf-8") as outfile:
        outfile.write(pydantic_model.json())

    return file_path



from nwon_baseline.shell_helper.execute_command import (
    execute_command,
    execute_command_and_print,
)
from nwon_baseline.shell_helper.get_environment_variable import get_environment_variable

__all__ = ["execute_command", "execute_command_and_print", "get_environment_variable"]
